@@ -1,6 +1,6 @@
 #!python
-# Copyright (c) 2000-2006 ActiveState Software Inc.
-# See the file LICENSE.txt for licensing information.
+# Copyright (c) 2000-2008 ActiveState Software Inc.
+# License: MIT License (http://www.opensource.org/licenses/mit-license.php)
 
 """
     test suite harness
@@ -17,6 +17,7 @@
         -h, --help      print this text and exit
         -l, --list      Just list the available test modules. You can also
                         specify tags to play with module filtering.
+        -n, --no-default-tags   Ignore default tags
         -L <directive>  Specify a logging level via
                             <logname>:<levelname>
                         For example:
@@ -48,8 +49,7 @@
 # - See the optparse "TODO" below.
 # - Make the quiet option actually quiet.
 
-__revision__ = "$Id: testlib.py 139 2008-09-09 05:44:49Z trentm $"
-__version_info__ = (0, 5, 1)
+__version_info__ = (0, 6, 6)
 __version__ = '.'.join(map(str, __version_info__))
 
 
@@ -60,12 +60,8 @@ import sys
 import getopt
 import glob
 import time
-import types
-import tempfile
 import unittest
-from pprint import pprint
 import imp
-import optparse
 import logging
 import textwrap
 import traceback
@@ -113,7 +109,7 @@ def tag(*tags):
 #---- timedtest decorator
 # Use this to assert that a test completes in a given amount of time.
 # This is from http://www.artima.com/forums/flat.jsp?forum=122&thread=129497
-# Including here, becase it might be useful.
+# Including here, because it might be useful.
 # NOTE: Untested and I suspect some breakage.
 
 TOLERANCE = 0.05
@@ -144,7 +140,7 @@ def timedtest(max_time, tolerance=TOLERANCE):
 
 #---- module api
 
-class Test:
+class Test(object):
     def __init__(self, ns, testmod, testcase, testfn_name,
                  testsuite_class=None):
         self.ns = ns
@@ -239,20 +235,23 @@ def testmods_from_testdir(testdir):
         log.debug("import test module '%s'", testmod_path)
         try:
             iinfo = imp.find_module(testmod_name, [dirname(testmod_path)])
-            sys.path.insert(0, testdir)
+            testabsdir = abspath(testdir)
+            sys.path.insert(0, testabsdir)
             old_dir = os.getcwd()
             os.chdir(testdir)
             try:
                 testmod = imp.load_module(testmod_name, *iinfo)
             finally:
                 os.chdir(old_dir)
-                sys.path.remove(testdir)
-        except TestSkipped, ex:
-            log.warn("'%s' module skipped: %s", testmod_name, ex)
-        except Exception, ex:
-            log.warn("could not import test module '%s': %s (skipping, "
-                     "run with '-d' for full traceback)",
-                     testmod_path, ex)
+                sys.path.remove(testabsdir)
+        except TestSkipped:
+            _, ex, _ = sys.exc_info()
+            log.warning("'%s' module skipped: %s", testmod_name, ex)
+        except Exception:
+            _, ex, _ = sys.exc_info()
+            log.warning("could not import test module '%s': %s (skipping, "
+                        "run with '-d' for full traceback)",
+                        testmod_path, ex)
             if log.isEnabledFor(logging.DEBUG):
                 traceback.print_exc()
         else:
@@ -288,13 +287,14 @@ def testcases_from_testmod(testmod):
                     continue
                 for testcase in loader.loadTestsFromTestCase(testcase_class):
                     yield testcase
-        except Exception, ex:
+        except Exception:
+            _, ex, _ = sys.exc_info()
             testmod_path = testmod.__file__
             if testmod_path.endswith(".pyc"):
                 testmod_path = testmod_path[:-1]
-            log.warn("error running test_cases() in '%s': %s (skipping, "
-                     "run with '-d' for full traceback)",
-                     testmod_path, ex)
+            log.warning("error running test_cases() in '%s': %s (skipping, "
+                        "run with '-d' for full traceback)",
+                        testmod_path, ex)
             if log.isEnabledFor(logging.DEBUG):
                 traceback.print_exc()
     else:
@@ -333,21 +333,15 @@ def tests_from_manifest(testdir_from_ns):
                     testmod_path = testmod.__file__
                     if testmod_path.endswith(".pyc"):
                         testmod_path = testmod_path[:-1]
-                    log.warn("'test_suite_class' of '%s' module is not a "
-                             "subclass of 'unittest.TestSuite': ignoring",
-                             testmod_path)
+                    log.warning("'test_suite_class' of '%s' module is not a "
+                                "subclass of 'unittest.TestSuite': ignoring",
+                                testmod_path)
             else:
                 testsuite_class = None
             for testcase in testcases_from_testmod(testmod):
-                try:
-                    yield Test(ns, testmod, testcase,
-                               testcase._testMethodName,
-                               testsuite_class)
-                except AttributeError:
-                    # Python 2.4 and older:
-                    yield Test(ns, testmod, testcase,
-                               testcase._TestCase__testMethodName,
-                               testsuite_class)
+                yield Test(ns, testmod, testcase,
+                            testcase._testMethodName,
+                            testsuite_class)
 
 def tests_from_manifest_and_tags(testdir_from_ns, tags):
     include_tags = [tag.lower() for tag in tags if not tag.startswith('-')]
@@ -441,23 +435,23 @@ def list_tests(testdir_from_ns, tags):
     if log.isEnabledFor(logging.INFO): # long-form
         for i, t in enumerate(tests):
             if i:
-                print
+                print()
             testfile = t.testmod.__file__
             if testfile.endswith(".pyc"):
                 testfile = testfile[:-1]
-            print "%s:" % t.shortname()
-            print "  from: %s#%s.%s" \
-                  % (testfile, t.testcase.__class__.__name__, t.testfn_name)
+            print("%s:" % t.shortname())
+            print("  from: %s#%s.%s" % (testfile,
+                t.testcase.__class__.__name__, t.testfn_name))
             wrapped = textwrap.fill(' '.join(t.tags()), WIDTH-10)
-            print "  tags: %s" % _indent(wrapped, 8, True)
+            print("  tags: %s" % _indent(wrapped, 8, True))
             if t.doc():
-                print _indent(t.doc(), width=2)
+                print(_indent(t.doc(), width=2))
     else:
         for t in tests:
             line = t.shortname() + ' '
             if t.explicit_tags():
                 line += '[%s]' % ' '.join(t.explicit_tags())
-            print line
+            print(line)
 
 
 #---- text test runner that can handle TestSkipped reasonably
@@ -521,7 +515,7 @@ class ConsoleTestResult(unittest.TestResult):
             self.stream.write("%s\n" % err)
 
 
-class ConsoleTestRunner:
+class ConsoleTestRunner(object):
     """A test runner class that displays results on the console.
 
     It prints out the names of tests as they are run, errors as they
@@ -619,12 +613,13 @@ def _indent(s, width=4, skip_first_line=False):
 #
 #    return opts, raw_tags
 
-def _parse_opts(args):
+def _parse_opts(args, default_tags):
     """_parse_opts(args) -> (log_level, action, tags)"""
-    opts, raw_tags = getopt.getopt(args, "hvqdlL:",
-        ["help", "verbose", "quiet", "debug", "list"])
+    opts, raw_tags = getopt.getopt(args, "hvqdlL:n",
+        ["help", "verbose", "quiet", "debug", "list", "no-default-tags"])
     log_level = logging.WARN
     action = "test"
+    no_default_tags = False
     for opt, optarg in opts:
         if opt in ("-h", "--help"):
             action = "help"
@@ -636,6 +631,8 @@ def _parse_opts(args):
             log_level = logging.DEBUG
         elif opt in ("-l", "--list"):
             action = "list"
+        elif opt in ("-n", "--no-default-tags"):
+            no_default_tags = True
         elif opt == "-L":
             # Optarg is of the form '<logname>:<levelname>', e.g.
             # "codeintel:DEBUG", "codeintel.db:INFO".
@@ -644,7 +641,10 @@ def _parse_opts(args):
             logging.getLogger(lname).setLevel(llevel)
 
     # Clean up the given tags.
-    tags = []
+    if no_default_tags:
+        tags = []
+    else:
+        tags = default_tags
     for raw_tag in raw_tags:
         if splitext(raw_tag)[1] in (".py", ".pyc", ".pyo", ".pyw") \
            and exists(raw_tag):
@@ -662,7 +662,7 @@ def _parse_opts(args):
 
 
 def harness(testdir_from_ns={None: os.curdir}, argv=sys.argv,
-            setup_func=None):
+            setup_func=None, default_tags=None):
     """Convenience mainline for a test harness "test.py" script.
 
         "testdir_from_ns" (optional) is basically a set of directories in
@@ -675,6 +675,7 @@ def harness(testdir_from_ns={None: os.curdir}, argv=sys.argv,
         "setup_func" (optional) is a callable that will be called once
             before any tests are run to prepare for the test suite. It
             is not called if no tests will be run.
+        "default_tags" (optional)
     
     Typically, if you have a number of test_*.py modules you can create
     a test harness, "test.py", for them that looks like this:
@@ -684,16 +685,18 @@ def harness(testdir_from_ns={None: os.curdir}, argv=sys.argv,
             retval = testlib.harness()
             sys.exit(retval)
     """
-    logging.basicConfig()
+    if not logging.root.handlers:
+        logging.basicConfig()
     try:
-        log_level, action, tags = _parse_opts(argv[1:])
-    except getopt.error, ex:
+        log_level, action, tags = _parse_opts(argv[1:], default_tags or [])
+    except getopt.error:
+        _, ex, _ = sys.exc_info()
         log.error(str(ex) + " (did you need a '--' before a '-TAG' argument?)")
         return 1
     log.setLevel(log_level)
 
     if action == "help":
-        print __doc__
+        print(__doc__)
         return 0
     if action == "list":
         return list_tests(testdir_from_ns, tags)
